import { NodeInfoOverlay } from "./components/nodeinfo";

import React from "react";
import screenfull from "screenfull";
import { GraphRenderer } from "./renderer";
import * as Helpers from "./helpers";
import Graph, { NodeData } from "./graph";
import { loadGraphJson } from "../datasets";
import PropTypes, { InferType } from "prop-types";
import NodeInfoBar from "./components/nodeinfo/nodeinfobar";

class Display extends React.Component<
    InferType<typeof Display.propTypes>,
    InferType<typeof Display.stateTypes>
> {
    props: InferType<typeof Display.propTypes>;
    state: InferType<typeof Display.stateTypes>;

    infoOverlay: NodeInfoOverlay;
    fullscreenRef: React.RefObject<HTMLDivElement>;

    static propTypes = {
        spaceId: PropTypes.string.isRequired,
    };

    static stateTypes = {
        graph: PropTypes.instanceOf(Graph),
        currentNode: PropTypes.object,
        nodeActive: PropTypes.bool,
        width: PropTypes.number,
        height: PropTypes.number,
    };

    constructor(props: InferType<typeof Display.propTypes>) {
        super(props);
        this.fullscreenRef = React.createRef();
        this.state = {
            graph: null,
            currentNode: null,
            nodeActive: false,
            width: 0,
            height: 0,
        };
        this.handleNodeClicked = this.handleNodeClicked.bind(this);
        this.handleNodeClose = this.handleNodeClose.bind(this);
    }

    componentDidMount() {
        this.setState({
            width: Helpers.getWidth(),
            height: Helpers.getHeight(),
        });

        const fetchGraph = async () => {
            const graphData = await loadGraphJson(this.props.spaceId);
            const graph = new Graph(graphData.nodes, graphData.links);
            this.setState({ graph: graph });
        };
        fetchGraph();
    }

    handleNodeClicked(node: NodeData) {
        this.setState({ currentNode: node, nodeActive: true });
    }

    handleNodeClose() {
        this.setState({ nodeActive: false });
    }

    toggleFullscreen() {
        if (screenfull.isEnabled) {
            if (!screenfull.isFullscreen) {
                this.setState({
                    width: screen.width,
                    height: screen.height,
                });
            } else {
                this.setState({
                    width: Helpers.getWidth(),
                    height: Helpers.getHeight(),
                });
            }

            screenfull.toggle(this.fullscreenRef.current);
        } else {
            console.log("No fullscreen mode available :(");
        }
    }

    render() {
        return (
            <div
                id="kg-display"
                style={{ position: "relative" }}
                ref={this.fullscreenRef}
            >
                <div
                    className={"fullscreen-button no-select"}
                    title={"Vollbild"}
                    onClick={this.toggleFullscreen.bind(this)}
                >
                    <p>&#10530;</p>
                </div>
                {/*{this.graph && (*/}
                {/*    <FilterOverlay graph={this.graph} type={"node"} />*/}
                {/*)}*/}

                {this.state.currentNode && (
                    <NodeInfoBar
                        node={this.state.currentNode}
                        height={this.state.nodeActive ? this.state.height : 0}
                        nodeClosedCallback={this.handleNodeClose}
                    ></NodeInfoBar>
                )}

                <div id="3d-graph">
                    {this.state.graph && (
                        <GraphRenderer
                            graph={this.state.graph}
                            width={this.state.width}
                            height={this.state.height}
                            onNodeClicked={this.handleNodeClicked}
                        />
                    )}
                </div>
            </div>
        );
    }
}

export default Display;
